'' =================================================================================================
''
''   File....... jm_hm-10_bridge.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2020 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 25 AUG 2020
''
''   {$P1} 
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  BR_TERM  = 115_200                                            ' terminal baud rate
  BR_BLE   =   9_600                                            ' ble baud rate
  

con { fixed io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                             
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O }


con { app io pins }                                                  
                                                                 
  RX2  =  5  { I }                                              ' ble
  TX2  =  4  { O } 


con

  #true,  ON, OFF 
  #false, NO, YES

  BUF_SIZE = 40


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  io   : "jm_io_basic"                                          '   essential io
  tbuf : "jm_textbuf"                                           '   text buffer for commands
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ble  : "jm_fullduplexserial"                                  ' * serial IO for hm-10 ble      
                                                                 
' * uses cog when loaded

                                                                 
dat { pre-initialized }


var { globals }

  byte  buffer[BUF_SIZE]                                                             
                                                                 
                                                                 
pub main | c
                                                                 
  setup

  time.pause(10)
  term.rxflush
  ble.rxflush

  repeat
    check_command
    check_response


pub check_command | c

  repeat
    c := term.rxtime(50)
    if (c => 0)
      tbuf.clear
      tbuf.putc(tbuf.upper(c))
      repeat
        c := term.rx
        if (c <> 13)
          tbuf.putc(tbuf.upper(c))
        else
          term.fstr1(string("--> %s\r"), tbuf.str)
          ble.str(tbuf.str)
          return


pub check_response | c

  c := ble.rxcheck
  if (c < 0)
    return

  term.fstr1(string("<-- %c"), c)

  repeat
    c := ble.rxcheck   
    if (c < 32)
      term.tx(13)
      return
    else
      term.tx(c)

                                                               
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  tbuf.use(@buffer, BUF_SIZE-1)                                 ' connect buffer
  tbuf.clear
                                                                 
  term.start(RX1, TX1, %0000, BR_TERM)                          ' start serial for terminal *

  ble.start(RX2, TX2, %0000, BR_BLE)                            ' start serial for hm-10 ble *

                   
con { license }                                                  

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}     